local Tags={}
Relog.Tags=Tags

local TagCompiler = AceLibrary("TagCompiler-1.0")

local PowerColors={}
local ClassColors={}

for i=0,4 do
	PowerColors[i]=string.format("|cff%02x%02x%02x",255*ManaBarColor[i].r,255*ManaBarColor[i].g,255*ManaBarColor[i].b)
end

for k,v in pairs(RAID_CLASS_COLORS) do
	ClassColors[k]=string.format("|cff%02x%02x%02x",255*v.r,255*v.g,255*v.b)
end

local TagData = {
	Time = { tableData = "Time" },
	Type = { tableData = "Type" },
	Source = { tableData = "Source" },
	Victim = { tableData = "Victim" },
	Ability = { tableData = "Ability" },
	Result = { tableData = "Result" },
	Amount = { tableData = "Amount" },

	Attribute ={ tableData = "Attribute"},

	--Leech Info
	SourceGained={ tableData = "SourceGained"},
	AttributeGained={ tableData = "AttributeGained"},

	--Reputation
	Faction={ tableData = "Faction"},
	Rank={ tableData = "Rank"},

	--Special Data for Damage
	DamageType={ tableData = "DamageType"},
	AbsorbAmount={ tableData = "AbsorbAmount"},
	BlockAmount={ tableData = "BlockAmount"},
	ResistAmount={ tableData = "ResistAmount"},
	VulnerableAmount={ tableData = "VulnerableAmount"},

	--Healing
	Overheal={ tableData = "Overheal"},

	--Item Data
	ItemLink={ tableData = "ItemLink"},
	ItemName={ tableData = "ItemName"},
	ItemRarity={ tableData = "ItemRarity"},
	ItemString={ tableData = "ItemString"},

	--Items just for Experience which requires several
	BonusAmount={  tableData = "BonusAmount"},
	BonusType={ tableData = "BonusType"},
	PenaltyAmount={ tableData = "PenaltyAmount"},
	PenaltyType={ tableData = "PenaltyType"},
	AmountRaidPenalty={ tableData = "AmountRaidPenalty"},
	AmountGroupBonus={ tableData = "AmountGroupBonus"},


	--Unit Functions
	UnitID = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.UnitID end },
	Friendly = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Friendly end },
	isPlayer = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.isPlayer end },
	isGrouped = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.isGrouped end },
	Class = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.lClass end },
	Faction = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Faction end },
	Level = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Level end },

	Health = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Health end },
	HealthMax = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.HealthMax end },
	Power = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Power end },
	PowerMax = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.PowerMax end },
	PowerType = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.PowerType end },
	MinRange = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.MinRange end },
	MaxRange = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.MaxRange end },

	RankName = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.RankName end },
	RankNumber = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.RankNumber end },

	FriendlyC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Friendly then return "|cff40ff40"..prior.."|r";else return "|cffff4040"..prior.."|r" end end },
	ClassC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Class then return ClassColors[U.Class]..prior.."|r";else return prior end end },

	--Start Color Commands
	StartFriendlyC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Friendly then return "|cff40ff40";else return "|cffff4040" end end },
	StartClassC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Class then return ClassColors[U.Class];else return "" end end },
	StartPowerC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.PowerType then return PowerColors[U.PowerType];else return "" end end },

	EndC = { string = "|r" },
}

local DefaultTags={
	Buff = "[Victim] is buffed with [Ability][Amount>1:\"(\"][Amount:hideIf(Amount <= 1)][Amount>1:\")\"]",
	Debuff = "[Victim] is debuffed with [Ability][Amount>1:\"(\"][Amount:hideIf(Amount <= 1)][Amount>1:\")\"]",
	Cast = "[Source] [Result:\"begins to \"][Amount:\"perform \"] [Ability] on [Victim]",
	Damage = "[Source:FriendlyC..\"(\"..Health..\"/\"..HealthMax..\")\"] [Result] [Victim:FriendlyC..\"(\"..Health..\"/\"..HealthMax..\")\"] for [Amount] [DamageType:DamageType]",
	Death = "[Victim:FriendlyC] killed",-- [Source:\"by \"..ClassC]",--[Ability:\"'s \"..Ability]",
	Environment = "[Victim] [Result] for [Amount] [DamageType]",
}

local CurTime
function Relog:TestTag(num)
	--Relog:Print(TagCompiler:Evaluate("[Type] [Source] [Result] [Victim]", TagData, "string", Relog:GetLog(num)))
	local log=Relog:GetLog(num)

	CurTime=log.Time
	Tags.ResetCache()

	if DefaultTags[log.Type] then
		Relog:CustomPrint(1,1,1,ChatFrame3,nil,nil,TagCompiler:Evaluate(DefaultTags[log.Type], TagData, "string", log))
	else
		Relog:CustomPrint(1,1,1,ChatFrame3,nil,nil,"No String for "..log.Type)
	end
end

local UnitCache={}
local NumInCache

function Tags.ResetCache()
	for k in pairs(UnitCache) do
		UnitCache[k]=nil
	end

	NumInCache=0
end

local Previous

function Tags.GetUnit(name)
	if name==true then
		name=Previous
	else
		Previous=name
	end

	local Unit=UnitCache[name]

	if Unit==nil then
		NumInCache=NumInCache+1
		UnitCache[name]=Relog.Units:GetUnit(name,CurTime,NumInCache)
		Unit=UnitCache[name]
	end

	return Unit	
end